"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const governance_1 = require("@celo/governance");
const async_1 = require("@celo/utils/lib/async");
const util_1 = require("@ethereumjs/util");
const command_1 = require("@oclif/command");
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = require("fs");
const base_1 = require("../../base");
const checks_1 = require("../../utils/checks");
const cli_1 = require("../../utils/cli");
const governance_2 = require("../../utils/governance");
class Show extends base_1.BaseCommand {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = this.parse(Show);
            const id = res.flags.proposalID;
            const raw = res.flags.raw;
            const account = res.flags.account;
            const hotfix = res.flags.hotfix;
            const governance = yield this.kit.contracts.getGovernance();
            if (id) {
                yield (0, checks_1.newCheckBuilder)(this).proposalExists(id).runChecks();
                const record = yield governance.getProposalRecord(id);
                const proposal = record.proposal;
                if (!raw) {
                    const builder = new governance_1.ProposalBuilder(this.kit);
                    if (res.flags.afterExecutingID) {
                        yield (0, governance_2.addExistingProposalIDToBuilder)(this.kit, builder, res.flags.afterExecutingID);
                    }
                    else if (res.flags.afterExecutingProposal) {
                        yield (0, governance_2.addExistingProposalJSONFileToBuilder)(builder, res.flags.afterExecutingProposal);
                    }
                    try {
                        const jsonproposal = yield (0, governance_1.proposalToJSON)(this.kit, proposal, builder.registryAdditions);
                        record.proposal = jsonproposal;
                        if (res.flags.jsonTransactions) {
                            console.log(chalk_1.default.yellowBright(`Outputting proposal JSON to ${res.flags.jsonTransactions}`));
                            (0, fs_1.writeFileSync)(res.flags.jsonTransactions, JSON.stringify(jsonproposal, null, 2));
                        }
                    }
                    catch (error) {
                        console.warn(`Could not decode proposal, displaying raw data: ${error}`);
                    }
                }
                let requirements = {};
                if (record.stage === 'Referendum' || record.stage === 'Execution') {
                    // Identify the transaction with the highest constitutional requirement.
                    const constitutionThreshold = yield governance.getConstitution(proposal);
                    const support = yield governance.getSupportWithConstitutionThreshold(id, constitutionThreshold);
                    requirements = Object.assign({ constitutionThreshold }, support);
                }
                const schedule = yield governance.humanReadableProposalSchedule(id);
                (0, cli_1.printValueMapRecursive)(Object.assign(Object.assign({}, record), { schedule }));
                if (Object.keys(requirements).length !== 0) {
                    console.log('Note: required is the minimal amount of yes + abstain votes needed to pass the proposal');
                    (0, cli_1.printValueMapRecursive)({
                        requirements,
                    });
                }
            }
            else if (hotfix) {
                const hotfixBuf = (0, util_1.toBuffer)(hotfix);
                const record = yield governance.getHotfixRecord(hotfixBuf);
                (0, cli_1.printValueMap)(record);
                const passing = yield governance.isHotfixPassing(hotfixBuf);
                (0, cli_1.printValueMap)({ passing });
                const tally = yield governance.hotfixWhitelistValidatorTally(hotfixBuf);
                const quorum = yield governance.minQuorumSize();
                (0, cli_1.printValueMap)({
                    tally,
                    quorum,
                });
                if (res.flags.whitelisters || res.flags.nonwhitelisters) {
                    const validators = yield this.kit.contracts.getValidators();
                    const accounts = yield validators.currentValidatorAccountsSet();
                    const whitelist = yield (0, async_1.concurrentMap)(5, accounts, (validator) => __awaiter(this, void 0, void 0, function* () {
                        return (yield governance.isHotfixWhitelistedBy(hotfixBuf, validator.signer)) ||
                            /* tslint:disable-next-line no-return-await */
                            (yield governance.isHotfixWhitelistedBy(hotfixBuf, validator.account));
                    }));
                    (0, cli_1.printValueMapRecursive)({
                        Validators: accounts.filter((_, idx) => !!res.flags.whitelisters === whitelist[idx]),
                    });
                }
            }
            else if (account) {
                const accounts = yield this.kit.contracts.getAccounts();
                (0, cli_1.printValueMapRecursive)(yield governance.getVoter(yield accounts.signerToAccount(account)));
            }
        });
    }
}
exports.default = Show;
Show.aliases = [
    'governance:show',
    'governance:showhotfix',
    'governance:showaccount',
    'governance:view',
    'governance:viewhotfix',
    'governance:viewaccount',
];
Show.description = 'Show information about a governance proposal, hotfix, or account.';
Show.flags = Object.assign(Object.assign({}, base_1.BaseCommand.flags), { raw: command_1.flags.boolean({ required: false, description: 'Display proposal in raw bytes format' }), jsonTransactions: command_1.flags.string({
        required: false,
        description: 'Output proposal JSON to provided file',
    }), proposalID: command_1.flags.string({
        exclusive: ['account', 'hotfix'],
        description: 'UUID of proposal to view',
    }), account: command_1.flags.string({
        exclusive: ['proposalID', 'hotfix'],
        description: 'Address of account or voter',
    }), hotfix: command_1.flags.string({
        exclusive: ['account', 'proposalID'],
        description: 'Hash of hotfix proposal',
    }), notwhitelisted: command_1.flags.boolean({
        description: 'List validators who have not whitelisted the specified hotfix',
    }), whitelisters: command_1.flags.boolean({
        description: 'If set, displays validators that have whitelisted the hotfix.',
        exclusive: ['nonwhitelisters', 'account', 'proposalID'],
    }), nonwhitelisters: command_1.flags.boolean({
        description: 'If set, displays validators that have not whitelisted the hotfix.',
        exclusive: ['whitelisters', 'account', 'proposalID'],
    }), afterExecutingProposal: command_1.flags.string({
        required: false,
        description: 'Path to proposal which will be executed prior to proposal',
        exclusive: ['afterExecutingID'],
    }), afterExecutingID: command_1.flags.string({
        required: false,
        description: 'Governance proposal identifier which will be executed prior to proposal',
        exclusive: ['afterExecutingProposal'],
    }) });
Show.examples = [
    'show --proposalID 99',
    'show --proposalID 99 --raw',
    'show --hotfix 0x614dccb5ac13cba47c2430bdee7829bb8c8f3603a8ace22e7680d317b39e3658',
    'show --hotfix 0x614dccb5ac13cba47c2430bdee7829bb8c8f3603a8ace22e7680d317b39e3658 --whitelisters',
    'show --hotfix 0x614dccb5ac13cba47c2430bdee7829bb8c8f3603a8ace22e7680d317b39e3658 --nonwhitelisters',
    'show --account 0x47e172f6cfb6c7d01c1574fa3e2be7cc73269d95',
];
//# sourceMappingURL=show.js.map