"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const identity_1 = require("@celo/contractkit/lib/identity");
const async_1 = require("@celo/utils/lib/async");
const string_1 = require("@celo/utils/lib/string");
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const base_1 = require("../../base");
const checks_1 = require("../../utils/checks");
const command_2 = require("../../utils/command");
class TestAttestationService extends base_1.BaseCommand {
    constructor() {
        super(...arguments);
        this.requireSynced = false;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(TestAttestationService);
            const address = flags.from;
            const { phoneNumber, message, provider } = flags;
            yield (0, checks_1.newCheckBuilder)(this, flags.from).isSignerOrAccount().canSign(address).runChecks();
            const accounts = yield this.kit.contracts.getAccounts();
            const account = yield accounts.signerToAccount(address);
            const hasAuthorizedAttestationSigner = yield accounts.hasAuthorizedAttestationSigner(account);
            if (!hasAuthorizedAttestationSigner) {
                console.info('Account has not authorized an attestation signer');
                return;
            }
            const metadataURL = yield accounts.getMetadataURL(account);
            if (!metadataURL) {
                console.info('No metadata set for address');
                return;
            }
            let metadata;
            try {
                metadata = yield identity_1.IdentityMetadataWrapper.fetchFromURL(accounts, metadataURL);
            }
            catch (error) {
                console.error(`Metadata could not be retrieved from ${metadataURL}: ${error.toString()}`);
                return;
            }
            const attestationServiceUrlClaim = metadata.findClaim(identity_1.ClaimTypes.ATTESTATION_SERVICE_URL);
            if (!attestationServiceUrlClaim) {
                console.error('No attestation service claim could be found');
                return;
            }
            const signature = yield this.kit.connection.sign(phoneNumber + message, address);
            try {
                const testUrl = (0, string_1.appendPath)(attestationServiceUrlClaim.url, 'test_attestations');
                cli_ux_1.cli.action.start(`Sending request to ${testUrl}`);
                console.info();
                const response = yield (0, cross_fetch_1.default)(testUrl, {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ phoneNumber, signature, message, provider }),
                });
                if (!response.ok) {
                    console.error('Request was not successful');
                    console.error(`Status: ${response.status}`);
                    console.error(`Response: ${yield response.text()}`);
                }
                cli_ux_1.cli.action.stop();
                const testRes = JSON.parse(yield response.text());
                if (!testRes.success) {
                    console.error('Request was not successful');
                    cli_ux_1.cli.styledJSON(testRes);
                    return;
                }
                console.info('Request successful');
                if (testRes.salt) {
                    // Service supports tracking attestation delivery status.
                    const getUrl = (0, string_1.appendPath)(attestationServiceUrlClaim.url, 'get_attestations');
                    cli_ux_1.cli.action.start(`Checking for delivery status at ${getUrl}`);
                    let latestGet = null;
                    for (let i = 0; i < 6; i++) {
                        yield (0, async_1.sleep)(5 * 1000);
                        const getResponse = yield (0, cross_fetch_1.default)(getUrl +
                            '?' +
                            new URLSearchParams({
                                phoneNumber,
                                salt: testRes.salt,
                                issuer: testRes.issuer,
                                account: testRes.account,
                            }), {
                            method: 'GET',
                            headers: { 'Content-Type': 'application/json' },
                        });
                        if (!getResponse.ok) {
                            console.error('Request was not successful');
                            console.error(`Status: ${getResponse.status}`);
                            console.error(`Response: ${yield getResponse.text()}`);
                        }
                        latestGet = JSON.parse(yield getResponse.text());
                        if (latestGet.status === 'Delivered') {
                            break;
                        }
                    }
                    cli_ux_1.cli.action.stop();
                    if (latestGet) {
                        cli_ux_1.cli.styledJSON(latestGet);
                    }
                }
            }
            catch (error) {
                console.error(`Something went wrong`);
                console.error(error);
            }
        });
    }
}
exports.default = TestAttestationService;
TestAttestationService.description = 'Tests whether the account has setup the attestation service properly by calling the test endpoint on it';
TestAttestationService.flags = Object.assign(Object.assign({}, base_1.BaseCommand.flags), { from: command_2.Flags.address({
        required: true,
        description: "Your validator's signer or account address",
    }), phoneNumber: command_2.Flags.phoneNumber({
        required: true,
        description: 'The phone number to send the test message to',
    }), message: command_1.flags.string({ required: true, description: 'The message of the SMS' }), provider: command_1.flags.string({
        required: false,
        description: 'Test a specific provider (try "twilio" or "nexmo")',
    }) });
TestAttestationService.examples = ['test-attestation-service --from 0x97f7333c51897469E8D98E7af8653aAb468050a3'];
//# sourceMappingURL=test-attestation-service.js.map