"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const address_1 = require("@celo/utils/lib/address");
const command_1 = require("@oclif/command");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const checks_1 = require("../../utils/checks");
const cli_1 = require("../../utils/cli");
const command_2 = require("../../utils/command");
const release_gold_base_1 = require("../../utils/release-gold-base");
class LockedGold extends release_gold_base_1.ReleaseGoldBaseCommand {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            // tslint:disable-next-line
            const { flags } = this.parse(LockedGold);
            const value = new bignumber_js_1.default(flags.value);
            const checkBuilder = (0, checks_1.newCheckBuilder)(this, this.contractAddress).isAccount(this.contractAddress);
            const isRevoked = yield this.releaseGoldWrapper.isRevoked();
            const beneficiary = yield this.releaseGoldWrapper.getBeneficiary();
            const releaseOwner = yield this.releaseGoldWrapper.getReleaseOwner();
            const lockedGold = yield this.kit.contracts.getLockedGold();
            this.kit.defaultAccount = isRevoked ? releaseOwner : beneficiary;
            if (flags.action === 'lock') {
                // Must verify contract is account before checking pending withdrawals
                yield checkBuilder.addCheck('Is not revoked', () => !isRevoked).runChecks();
                const pendingWithdrawalsValue = yield lockedGold.getPendingWithdrawalsTotalValue(this.contractAddress);
                const relockValue = bignumber_js_1.default.minimum(pendingWithdrawalsValue, value);
                const lockValue = value.minus(relockValue);
                yield (0, checks_1.newCheckBuilder)(this, this.contractAddress)
                    .hasEnoughCelo(this.contractAddress, lockValue)
                    .runChecks();
                const txos = yield this.releaseGoldWrapper.relockGold(relockValue);
                for (const txo of txos) {
                    yield (0, cli_1.displaySendTx)('lockedGoldRelock', txo, { from: beneficiary });
                }
                if (lockValue.gt(new bignumber_js_1.default(0))) {
                    const accounts = yield this.kit.contracts.getAccounts();
                    const totalValue = yield this.releaseGoldWrapper.getRemainingUnlockedBalance();
                    const remaining = totalValue.minus(lockValue);
                    console.log('remaining', remaining.toFixed());
                    if (!flags.yes &&
                        remaining.lt(new bignumber_js_1.default(2e18)) &&
                        ((0, address_1.eqAddress)(yield accounts.getVoteSigner(flags.contract), flags.contract) ||
                            (0, address_1.eqAddress)(yield accounts.getValidatorSigner(flags.contract), flags.contract))) {
                        const check = yield (0, cli_1.binaryPrompt)(`Only ${remaining.shiftedBy(-18)} CELO would be left unlocked, you might not be able to fund your signers. Unlock anyway?`, true);
                        if (!check) {
                            console.log('Cancelled');
                            return;
                        }
                    }
                    yield (0, cli_1.displaySendTx)('lockedGoldLock', this.releaseGoldWrapper.lockGold(lockValue));
                }
            }
            else if (flags.action === 'unlock') {
                yield checkBuilder
                    .isNotVoting(this.contractAddress)
                    .hasEnoughLockedGoldToUnlock(value)
                    .runChecks();
                yield (0, cli_1.displaySendTx)('lockedGoldUnlock', this.releaseGoldWrapper.unlockGold(flags.value));
            }
            else if (flags.action === 'withdraw') {
                yield checkBuilder.runChecks();
                const currentTime = Math.round(new Date().getTime() / 1000);
                while (true) {
                    let madeWithdrawal = false;
                    const pendingWithdrawals = yield lockedGold.getPendingWithdrawals(this.contractAddress);
                    for (let i = 0; i < pendingWithdrawals.length; i++) {
                        const pendingWithdrawal = pendingWithdrawals[i];
                        if (pendingWithdrawal.time.isLessThan(currentTime)) {
                            console.log(`Found available pending withdrawal of value ${pendingWithdrawal.value.toFixed()}, withdrawing`);
                            yield (0, cli_1.displaySendTx)('lockedGoldWithdraw', this.releaseGoldWrapper.withdrawLockedGold(i));
                            madeWithdrawal = true;
                            break;
                        }
                    }
                    if (!madeWithdrawal)
                        break;
                }
                const remainingPendingWithdrawals = yield lockedGold.getPendingWithdrawals(this.contractAddress);
                for (const pendingWithdrawal of remainingPendingWithdrawals) {
                    console.log(`Pending withdrawal of value ${pendingWithdrawal.value.toFixed()} available for withdrawal in ${pendingWithdrawal.time
                        .minus(currentTime)
                        .toFixed()} seconds.`);
                }
            }
        });
    }
}
exports.default = LockedGold;
LockedGold.description = 'Perform actions [lock, unlock, withdraw] on CELO that has been locked via the provided ReleaseGold contract.';
LockedGold.flags = Object.assign(Object.assign({}, release_gold_base_1.ReleaseGoldBaseCommand.flags), { action: command_1.flags.string({
        char: 'a',
        options: ['lock', 'unlock', 'withdraw'],
        description: "Action to perform on contract's gold",
        required: true,
    }), value: command_2.Flags.wei({ required: true, description: 'Amount of gold to perform `action` with' }), yes: command_1.flags.boolean({ description: 'Answer yes to prompt' }) });
LockedGold.examples = [
    'locked-gold --contract 0xCcc8a47BE435F1590809337BB14081b256Ae26A8 --action lock --value 10000000000000000000000',
    'locked-gold --contract 0xCcc8a47BE435F1590809337BB14081b256Ae26A8 --action unlock --value 10000000000000000000000',
    'locked-gold --contract 0xCcc8a47BE435F1590809337BB14081b256Ae26A8 --action withdraw --value 10000000000000000000000',
];
//# sourceMappingURL=locked-gold.js.map