"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.statusTable = void 0;
const address_1 = require("@celo/utils/lib/address");
const async_1 = require("@celo/utils/lib/async");
const istanbul_1 = require("@celo/utils/lib/istanbul");
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const base_1 = require("../../base");
const checks_1 = require("../../utils/checks");
const command_2 = require("../../utils/command");
const election_1 = require("../../utils/election");
exports.statusTable = {
    address: {},
    name: {},
    signer: {},
    elected: {},
    frontRunner: {},
    signatures: {
        get: (v) => isNaN(v.signatures) ? '' : (v.signatures * 100).toFixed(2) + '%',
    },
};
class ValidatorStatus extends base_1.BaseCommand {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = this.parse(ValidatorStatus);
            const accounts = yield this.kit.contracts.getAccounts();
            const validators = yield this.kit.contracts.getValidators();
            const election = yield this.kit.contracts.getElection();
            // Resolve the signer address(es) from the provide flags.
            let signers = [];
            const checker = (0, checks_1.newCheckBuilder)(this);
            if (res.flags.signer) {
                signers = [res.flags.signer];
                const validator = yield accounts.signerToAccount(res.flags.signer);
                checker.isAccount(validator).isValidator(validator);
                yield checker.runChecks();
            }
            else if (res.flags.validator) {
                checker.isAccount(res.flags.validator).isValidator(res.flags.validator);
                yield checker.runChecks();
                const signer = yield accounts.getValidatorSigner(res.flags.validator);
                signers = [signer];
            }
            else {
                signers = yield (0, async_1.concurrentMap)(10, yield validators.getRegisteredValidatorsAddresses(), (a) => accounts.getValidatorSigner(a));
            }
            const latest = yield this.web3.eth.getBlockNumber();
            const endBlock = res.flags.end === -1 ? yield this.web3.eth.getBlockNumber() : res.flags.end;
            const startBlock = res.flags.start === -1 ? endBlock - 100 : res.flags.start;
            if (startBlock > endBlock || endBlock > latest) {
                this.error('invalid values for start/end');
            }
            const epochSize = (yield validators.getEpochSize()).toNumber();
            const electionCache = new election_1.ElectionResultsCache(election, epochSize);
            let frontRunnerSigners = [];
            cli_ux_1.cli.action.start(`Running mock election`);
            try {
                frontRunnerSigners = yield election.electValidatorSigners();
            }
            catch (err) {
                console.warn('Warning: Elections not available');
            }
            cli_ux_1.cli.action.stop();
            const signatureCounts = yield this.getSignatureCounts(startBlock, endBlock, electionCache);
            const electedCounts = yield this.getElectedCounts(startBlock, endBlock, electionCache, epochSize);
            cli_ux_1.cli.action.start(`Fetching validator information`);
            const validatorStatuses = yield (0, async_1.concurrentMap)(10, signers, (s) => this.getStatus(s, signatureCounts, electedCounts, electionCache, frontRunnerSigners));
            cli_ux_1.cli.action.stop();
            cli_ux_1.cli.table(validatorStatuses, exports.statusTable, res.flags);
        });
    }
    getSignatureCounts(start, end, electionCache) {
        return __awaiter(this, void 0, void 0, function* () {
            const bar = cli_ux_1.cli.progress({
                format: 'counting block signatures [{bar}] {percentage}% | ETA: {eta}s | {value}/{total}',
            });
            bar.start(end - start);
            const countsBySigner = new Map();
            const incrementSignatureCounts = (blockNumber) => __awaiter(this, void 0, void 0, function* () {
                if ((blockNumber - start) % 10 === 0 || blockNumber === end) {
                    bar.update(blockNumber - start);
                }
                const block = yield this.web3.eth.getBlock(blockNumber);
                const bitmap = (0, istanbul_1.parseBlockExtraData)(block.extraData).parentAggregatedSeal.bitmap;
                const signers = yield electionCache.electedSigners(blockNumber);
                signers.map((s, i) => {
                    if ((0, istanbul_1.bitIsSet)(bitmap, i)) {
                        const count = countsBySigner.get(s) === undefined ? 0 : countsBySigner.get(s);
                        countsBySigner.set(s, count + 1);
                    }
                });
            });
            yield (0, async_1.concurrentMap)(10, Array.from({ length: end - start + 1 }, (_, i) => i + start), incrementSignatureCounts);
            const signerToAccountCache = new Map();
            const accounts = yield this.kit.contracts.getAccounts();
            yield (0, async_1.concurrentMap)(10, Array.from(countsBySigner.keys()), (signer) => __awaiter(this, void 0, void 0, function* () {
                const account = yield accounts.signerToAccount(signer);
                signerToAccountCache.set(signer, account);
            }));
            const countsByAccount = new Map();
            countsBySigner.forEach((count, signer) => __awaiter(this, void 0, void 0, function* () {
                const account = signerToAccountCache.get(signer);
                const total = countsByAccount.get(account);
                countsByAccount.set(account, total === undefined ? count : count + total);
            }));
            bar.stop();
            return countsByAccount;
        });
    }
    epochNumber(blockNumber, epochSize) {
        return Math.ceil(blockNumber / epochSize);
    }
    firstBlockOfEpoch(epochNumber, epochSize) {
        return (epochNumber - 1) * epochSize + 1;
    }
    getElectedCounts(start, end, electionCache, epochSize) {
        return __awaiter(this, void 0, void 0, function* () {
            const countsBySigner = new Map();
            let i = start;
            while (i < end) {
                const epoch = this.epochNumber(i, epochSize);
                const j = Math.min(this.firstBlockOfEpoch(epoch + 1, epochSize) - 1, end);
                const signers = yield electionCache.electedSigners(i);
                signers.map((s) => {
                    const count = countsBySigner.get(s) === undefined ? 0 : countsBySigner.get(s);
                    countsBySigner.set(s, count + (j - i) + 1);
                });
                i = j + 1;
            }
            const signerToAccountCache = new Map();
            const accounts = yield this.kit.contracts.getAccounts();
            yield (0, async_1.concurrentMap)(10, Array.from(countsBySigner.keys()), (signer) => __awaiter(this, void 0, void 0, function* () {
                const account = yield accounts.signerToAccount(signer);
                signerToAccountCache.set(signer, account);
            }));
            const countsByAccount = new Map();
            countsBySigner.forEach((count, signer) => __awaiter(this, void 0, void 0, function* () {
                const account = signerToAccountCache.get(signer);
                const total = countsByAccount.get(account);
                countsByAccount.set(account, total === undefined ? count : count + total);
            }));
            return countsByAccount;
        });
    }
    getStatus(signer, signatureCounts, electedCounts, electionCache, frontRunnerSigners) {
        return __awaiter(this, void 0, void 0, function* () {
            const accounts = yield this.kit.contracts.getAccounts();
            const validators = yield this.kit.contracts.getValidators();
            const validator = yield accounts.signerToAccount(signer);
            let name = 'Unregistered validator';
            if (yield validators.isValidator(validator)) {
                name = (yield accounts.getName(validator)) || '';
            }
            let signatures = signatureCounts.get(validator);
            if (signatures === undefined) {
                signatures = 0;
            }
            let elected = electedCounts.get(validator);
            if (elected === undefined) {
                elected = 0;
            }
            return {
                name,
                address: validator,
                signer,
                elected: yield electionCache.elected(signer, yield this.web3.eth.getBlockNumber()),
                frontRunner: frontRunnerSigners.some(address_1.eqAddress.bind(null, signer)),
                signatures: signatures / elected, // may be NaN
            };
        });
    }
}
exports.default = ValidatorStatus;
ValidatorStatus.description = 'Shows the consensus status of a validator. This command will show whether a validator is currently elected, would be elected if an election were to be run right now, and the percentage of blocks signed and number of blocks successfully proposed within a given window.';
ValidatorStatus.flags = Object.assign(Object.assign(Object.assign({}, base_1.BaseCommand.flags), { validator: command_2.Flags.address({
        description: 'address of the validator to check if elected and validating',
        exclusive: ['all', 'signer'],
    }), signer: command_2.Flags.address({
        description: 'address of the signer to check if elected and validating',
        exclusive: ['validator', 'all'],
    }), all: command_1.flags.boolean({
        description: 'get the status of all registered validators',
        exclusive: ['validator', 'signer'],
    }), start: command_1.flags.integer({
        description: 'what block to start at when looking at signer activity. defaults to the last 100 blocks',
        default: -1,
    }), end: command_1.flags.integer({
        description: 'what block to end at when looking at signer activity. defaults to the latest block',
        default: -1,
    }) }), cli_ux_1.cli.table.flags());
ValidatorStatus.examples = [
    'status --validator 0x5409ED021D9299bf6814279A6A1411A7e866A631',
    'status --validator 0x5409ED021D9299bf6814279A6A1411A7e866A631 --start 1480000',
    'status --all --start 1480000 --end 1490000',
];
//# sourceMappingURL=status.js.map