"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.newCheckBuilder = exports.check = void 0;
const address_1 = require("@celo/base/lib/address");
const contractkit_1 = require("@celo/contractkit");
const GrandaMento_1 = require("@celo/contractkit/lib/wrappers/GrandaMento");
const address_2 = require("@celo/utils/lib/address");
const signatureUtils_1 = require("@celo/utils/lib/signatureUtils");
const chalk_1 = __importDefault(require("chalk"));
const cli_1 = require("./cli");
function check(name, predicate, errorMessage) {
    return {
        name,
        errorMessage,
        run: predicate,
    };
}
exports.check = check;
const negate = (x) => x.then((y) => !y);
function newCheckBuilder(cmd, signer) {
    return new CheckBuilder(cmd, signer);
}
exports.newCheckBuilder = newCheckBuilder;
class CheckBuilder {
    constructor(cmd, signer) {
        this.cmd = cmd;
        this.signer = signer;
        this.checks = [];
        this.isApprover = (account) => this.addCheck(`${account} is approver address`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () { return (0, address_1.eqAddress)(yield governance.getApprover(), account); })));
        this.proposalExists = (proposalID) => this.addCheck(`${proposalID} is an existing proposal`, this.withGovernance((governance) => governance.proposalExists(proposalID)));
        this.grandaMentoProposalExists = (proposalID) => this.addCheck(`${proposalID} is an existing proposal`, this.withGrandaMento((grandaMento) => grandaMento.exchangeProposalExists(proposalID)));
        this.grandaMentoProposalHasState = (proposalID, state) => this.addCheck(`${proposalID} has state ${GrandaMento_1.ExchangeProposalState[state]}`, this.withGrandaMento((grandaMento) => __awaiter(this, void 0, void 0, function* () {
            const exchangeProposal = yield grandaMento.getExchangeProposal(proposalID);
            return exchangeProposal.state === state;
        })));
        this.grandaMentoProposalIsExecutable = (proposalID) => {
            this.grandaMentoProposalHasState(proposalID, GrandaMento_1.ExchangeProposalState.Approved);
            return this.addCheck(`${proposalID} veto period has elapsed`, this.withGrandaMento((grandaMento) => __awaiter(this, void 0, void 0, function* () {
                const exchangeProposal = yield grandaMento.getExchangeProposal(proposalID);
                return exchangeProposal.approvalTimestamp
                    .plus(exchangeProposal.vetoPeriodSeconds)
                    .isLessThanOrEqualTo(Date.now() / 1000);
            })));
        };
        this.proposalInStage = (proposalID, stage) => this.addCheck(`${proposalID} is in stage ${stage}`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () {
            const match = (yield governance.getProposalStage(proposalID)) === stage;
            if (!match) {
                const schedule = yield governance.proposalSchedule(proposalID);
                (0, cli_1.printValueMapRecursive)(schedule);
            }
            return match;
        })));
        this.proposalIsPassing = (proposalID) => this.addCheck(`Proposal ${proposalID} is passing corresponding constitutional quorum`, this.withGovernance((governance) => governance.isProposalPassing(proposalID)));
        this.hotfixIsPassing = (hash) => this.addCheck(`Hotfix 0x${hash.toString('hex')} is whitelisted by quorum of validators`, this.withGovernance((governance) => governance.isHotfixPassing(hash)));
        this.hotfixNotExecuted = (hash) => this.addCheck(`Hotfix 0x${hash.toString('hex')} is not already executed`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () { return !(yield governance.getHotfixRecord(hash)).executed; })));
        this.hotfixNotApproved = (hash) => this.addCheck(`Hotfix 0x${hash.toString('hex')} is not already approved`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () { return !(yield governance.getHotfixRecord(hash)).approved; })));
        this.canSign = (account) => this.addCheck('Account can sign', () => __awaiter(this, void 0, void 0, function* () {
            try {
                const message = 'test';
                const signature = yield this.kit.connection.sign(message, account);
                return (0, signatureUtils_1.verifySignature)(message, signature, account);
            }
            catch (error) {
                console.error(error);
                return false;
            }
        }));
        this.canSignValidatorTxs = () => this.addCheck('Signer can sign Validator Txs', this.withAccounts((accounts) => accounts
            .validatorSignerToAccount(this.signer)
            .then(() => true)
            .catch(() => false)));
        this.signerAccountIsValidator = () => this.addCheck(`Signer account is Validator`, this.withValidators((validators, _s, account) => validators.isValidator(account)));
        this.signerAccountIsValidatorGroup = () => this.addCheck(`Signer account is ValidatorGroup`, this.withValidators((validators, _s, account) => validators.isValidatorGroup(account)));
        this.isValidator = (account) => this.addCheck(`${account} is Validator`, this.withValidators((validators, _, _account) => validators.isValidator(account !== null && account !== void 0 ? account : _account)));
        this.isValidatorGroup = (account) => this.addCheck(`${account} is ValidatorGroup`, this.withValidators((validators) => validators.isValidatorGroup(account)));
        this.isNotValidator = (account) => this.addCheck(`${this.signer} is not a registered Validator`, this.withValidators((validators, _, _account) => negate(validators.isValidator(account !== null && account !== void 0 ? account : _account))));
        this.isNotValidatorGroup = () => this.addCheck(`${this.signer} is not a registered ValidatorGroup`, this.withValidators((validators, _, account) => negate(validators.isValidatorGroup(account))));
        this.signerMeetsValidatorBalanceRequirements = () => this.addCheck(`Signer's account has enough locked gold for registration`, this.withValidators((validators, _signer, account) => validators.meetsValidatorBalanceRequirements(account)));
        this.signerMeetsValidatorGroupBalanceRequirements = () => this.addCheck(`Signer's account has enough locked gold for group registration`, this.withValidators((validators, _signer, account) => validators.meetsValidatorGroupBalanceRequirements(account)));
        this.meetsValidatorBalanceRequirements = (account) => this.addCheck(`${account} has enough locked gold for registration`, this.withValidators((validators) => validators.meetsValidatorBalanceRequirements(account)));
        this.meetsValidatorGroupBalanceRequirements = (account) => this.addCheck(`${account} has enough locked gold for group registration`, this.withValidators((validators) => validators.meetsValidatorGroupBalanceRequirements(account)));
        this.isValidAddress = (address) => this.addCheck(`${address} is a valid address`, () => (0, address_2.isValidAddress)(address));
        this.isNotAccount = (address) => this.addCheck(`${address} is not a registered Account`, this.withAccounts((accounts) => negate(accounts.isAccount(address))));
        this.isSignerOrAccount = () => this.addCheck(`${this.signer} is Signer or registered Account`, this.withAccounts((accounts) => __awaiter(this, void 0, void 0, function* () {
            const res = (yield accounts.isAccount(this.signer)) || (yield accounts.isSigner(this.signer));
            return res;
        })), `${this.signer} is not a signer or registered as an account. Try authorizing as a signer or running account:register.`);
        this.isVoteSignerOrAccount = () => this.addCheck(`${this.signer} is vote signer or registered account`, this.withAccounts((accounts) => __awaiter(this, void 0, void 0, function* () {
            return accounts.voteSignerToAccount(this.signer).then((addr) => !(0, address_1.eqAddress)(addr, address_1.NULL_ADDRESS), () => false);
        })));
        this.isAccount = (address) => this.addCheck(`${address} is a registered Account`, this.withAccounts((accounts) => accounts.isAccount(address)), `${address} is not registered as an account. Try running account:register`);
        this.isNotVoting = (address) => this.addCheck(`${address} is not currently voting on a governance proposal`, this.withGovernance((governance) => negate(governance.isVoting(address))), `${address} is currently voting in governance. Revoke your upvotes or wait for the referendum to end.`);
        this.hasEnoughCelo = (account, value) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} CELO`, () => this.kit.contracts
                .getGoldToken()
                .then((goldToken) => goldToken.balanceOf(account))
                .then((balance) => balance.gte(value)));
        };
        this.hasEnoughStable = (account, value, stable = contractkit_1.StableToken.cUSD) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} ${stable}`, () => this.kit.contracts
                .getStableToken(stable)
                .then((stableToken) => stableToken.balanceOf(account))
                .then((balance) => balance.gte(value)));
        };
        this.hasEnoughErc20 = (account, value, erc20) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} erc20 token`, () => this.kit.contracts
                .getErc20(erc20)
                .then((goldToken) => goldToken.balanceOf(account))
                .then((balance) => balance.gte(value)));
        };
        this.exceedsProposalMinDeposit = (deposit) => this.addCheck(`Deposit is greater than or equal to governance proposal minDeposit`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () { return deposit.gte(yield governance.minDeposit()); })));
        this.hasRefundedDeposits = (account) => this.addCheck(`${account} has refunded governance deposits`, this.withGovernance((governance) => __awaiter(this, void 0, void 0, function* () { return !(yield governance.getRefundedDeposits(account)).isZero(); })));
        this.hasEnoughLockedGold = (value) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} Locked Gold`, this.withLockedGold((lockedGold, _signer, account) => __awaiter(this, void 0, void 0, function* () { return value.isLessThanOrEqualTo(yield lockedGold.getAccountTotalLockedGold(account)); })));
        };
        this.hasEnoughNonvotingLockedGold = (value) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} non-voting Locked Gold`, this.withLockedGold((lockedGold, _signer, account) => __awaiter(this, void 0, void 0, function* () { return value.isLessThanOrEqualTo(yield lockedGold.getAccountNonvotingLockedGold(account)); })));
        };
        this.hasEnoughLockedGoldToUnlock = (value) => {
            const valueInEth = this.kit.connection.web3.utils.fromWei(value.toFixed(), 'ether');
            return this.addCheck(`Account has at least ${valueInEth} non-voting Locked Gold over requirement`, this.withLockedGold((lockedGold, _signer, account, validators) => __awaiter(this, void 0, void 0, function* () {
                const requirement = yield validators.getAccountLockedGoldRequirement(account);
                return ((requirement.eq(0) ||
                    value.plus(requirement).lte(yield lockedGold.getAccountTotalLockedGold(account))) &&
                    value.lte(yield lockedGold.getAccountNonvotingLockedGold(account)));
            })));
        };
        this.isNotValidatorGroupMember = () => {
            return this.addCheck(`Account isn't a member of a validator group`, this.withValidators((validators, _signer, account) => __awaiter(this, void 0, void 0, function* () {
                const { affiliation } = yield validators.getValidator(account);
                if (!affiliation || (0, address_1.eqAddress)(affiliation, address_1.NULL_ADDRESS)) {
                    return true;
                }
                const { members } = yield validators.getValidatorGroup(affiliation);
                return !members.includes(account);
            })));
        };
        this.validatorDeregisterDurationPassed = () => {
            return this.addCheck(`Enough time has passed since the account was removed from a validator group`, this.withValidators((validators, _signer, account) => __awaiter(this, void 0, void 0, function* () {
                const { lastRemovedFromGroupTimestamp } = yield validators.getValidatorMembershipHistoryExtraData(account);
                const { duration } = yield validators.getValidatorLockedGoldRequirements();
                return duration.toNumber() + lastRemovedFromGroupTimestamp < Date.now() / 1000;
            })));
        };
        this.resetSlashingmultiplierPeriodPassed = () => {
            return this.addCheck(`Enough time has passed since the last halving of the slashing multiplier`, this.withValidators((validators, _signer, account) => __awaiter(this, void 0, void 0, function* () {
                const { lastSlashed } = yield validators.getValidatorGroup(account);
                const duration = yield validators.getSlashingMultiplierResetPeriod();
                return duration.toNumber() + lastSlashed.toNumber() < Date.now() / 1000;
            })));
        };
        this.hasACommissionUpdateQueued = () => this.addCheck("There's a commision update queued", this.withValidators((validators, _signer, account) => __awaiter(this, void 0, void 0, function* () {
            const vg = yield validators.getValidatorGroup(account);
            return !vg.nextCommissionBlock.eq(0);
        })));
        this.hasCommissionUpdateDelayPassed = () => this.addCheck('The Commission update delay has already passed', this.withValidators((validators, _signer, account, ctx) => __awaiter(this, void 0, void 0, function* () {
            const blockNumber = yield ctx.web3.eth.getBlockNumber();
            const vg = yield validators.getValidatorGroup(account);
            return vg.nextCommissionBlock.lte(blockNumber);
        })));
        this.isMultiSigOwner = (from, multisig) => {
            return this.addCheck('The provided address is an owner of the multisig', () => __awaiter(this, void 0, void 0, function* () {
                const owners = yield multisig.getOwners();
                return owners.indexOf(from) > -1;
            }));
        };
    }
    get web3() {
        return this.cmd.web3;
    }
    get kit() {
        return this.cmd.kit;
    }
    withValidators(f) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const validators = yield this.kit.contracts.getValidators();
            if (this.signer) {
                const account = yield validators.signerToAccount(this.signer);
                return f(validators, this.signer, account, this);
            }
            else {
                return f(validators, '', '', this);
            }
        });
    }
    withLockedGold(f) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const lockedGold = yield this.kit.contracts.getLockedGold();
            const validators = yield this.kit.contracts.getValidators();
            if (this.signer) {
                const account = yield validators.signerToAccount(this.signer);
                return f(lockedGold, this.signer, account, validators);
            }
            else {
                return f(lockedGold, '', '', validators);
            }
        });
    }
    withAccounts(f) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const accounts = yield this.kit.contracts.getAccounts();
            return f(accounts);
        });
    }
    withGrandaMento(f) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const accounts = yield this.kit.contracts.getGrandaMento();
            return f(accounts);
        });
    }
    withGovernance(f) {
        return () => __awaiter(this, void 0, void 0, function* () {
            const governance = yield this.kit.contracts.getGovernance();
            return f(governance, '', '', this);
        });
    }
    addCheck(name, predicate, errorMessage) {
        this.checks.push(check(name, predicate, errorMessage));
        return this;
    }
    addConditionalCheck(name, runCondition, predicate, errorMessage) {
        if (runCondition) {
            return this.addCheck(name, predicate, errorMessage);
        }
        return this;
    }
    runChecks() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(`Running Checks:`);
            let allPassed = true;
            for (const aCheck of this.checks) {
                const passed = yield aCheck.run();
                const status︎Str = chalk_1.default.bold(passed ? '✔' : '✘');
                const color = passed ? chalk_1.default.green : chalk_1.default.red;
                const msg = !passed && aCheck.errorMessage ? aCheck.errorMessage : '';
                console.log(color(`   ${status︎Str}  ${aCheck.name} ${msg}`));
                allPassed = allPassed && passed;
            }
            if (!allPassed) {
                return this.cmd.error("Some checks didn't pass!");
            }
        });
    }
}
//# sourceMappingURL=checks.js.map