"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Args = exports.Flags = exports.argBuilder = exports.parseIntRange = exports.parseAddressArray = exports.parsePath = void 0;
const io_1 = require("@celo/base/lib/io");
const contractkit_1 = require("@celo/contractkit");
const bls_1 = require("@celo/cryptographic-utils/lib/bls");
const phoneNumbers_1 = require("@celo/phone-utils/lib/phoneNumbers");
const address_1 = require("@celo/utils/lib/address");
const signatureUtils_1 = require("@celo/utils/lib/signatureUtils");
const command_1 = require("@oclif/command");
const errors_1 = require("@oclif/errors");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const fs_extra_1 = require("fs-extra");
const web3_1 = __importDefault(require("web3"));
const parseBytes = (input, length, msg) => {
    // Check that the string is hex and and has byte length of `length`.
    const expectedLength = input.startsWith('0x') ? length * 2 + 2 : length * 2;
    if (web3_1.default.utils.isHex(input) && input.length === expectedLength) {
        return (0, address_1.ensureLeading0x)(input);
    }
    else {
        throw new errors_1.CLIError(msg);
    }
};
const parseEcdsaPublicKey = (input) => {
    const stripped = (0, address_1.trimLeading0x)(input);
    // ECDSA public keys may be passed as 65 byte values. When this happens, we drop the first byte.
    if (stripped.length === 65 * 2) {
        return parseBytes(stripped.slice(2), 64, `${input} is not an ECDSA public key`);
    }
    else {
        return parseBytes(input, 64, `${input} is not an ECDSA public key`);
    }
};
const parseBlsPublicKey = (input) => {
    return parseBytes(input, bls_1.BLS_PUBLIC_KEY_SIZE, `${input} is not a BLS public key`);
};
const parseBlsProofOfPossession = (input) => {
    return parseBytes(input, bls_1.BLS_POP_SIZE, `${input} is not a BLS proof-of-possession`);
};
const parseProofOfPossession = (input) => {
    return parseBytes(input, signatureUtils_1.POP_SIZE, `${input} is not a proof-of-possession`);
};
const parseAddress = (input) => {
    if (web3_1.default.utils.isAddress(input)) {
        return input;
    }
    else {
        throw new errors_1.CLIError(`${input} is not a valid address`);
    }
};
const parseCoreContract = (input) => {
    if (contractkit_1.RegisteredContracts.includes(input)) {
        return input;
    }
    else {
        throw new errors_1.CLIError(`${input} is not a core contract`);
    }
};
const parseWei = (input) => {
    try {
        return new bignumber_js_1.default(input);
    }
    catch (_err) {
        throw new errors_1.CLIError(`${input} is not a valid token amount`);
    }
};
const parsePath = (input) => {
    if ((0, fs_extra_1.pathExistsSync)(input)) {
        return input;
    }
    else {
        throw new errors_1.CLIError(`File at "${input}" does not exist`);
    }
};
exports.parsePath = parsePath;
const parsePhoneNumber = (input) => {
    if ((0, phoneNumbers_1.isE164NumberStrict)(input)) {
        return input;
    }
    else {
        throw new errors_1.CLIError(`PhoneNumber "${input}" is not a valid E164 number`);
    }
};
const parseUrl = (input) => {
    if (io_1.URL_REGEX.test(input)) {
        return input;
    }
    else {
        throw new errors_1.CLIError(`"${input}" is not a valid URL`);
    }
};
function parseArray(parseElement) {
    return (input) => {
        const array = JSON.parse(input);
        if (Array.isArray(array)) {
            return array.map(parseElement);
        }
        else {
            throw new errors_1.CLIError(`"${input}" is not a valid array`);
        }
    };
}
exports.parseAddressArray = parseArray(parseAddress);
const parseIntRange = (input) => {
    const range = input
        .slice(1, input.length - 1)
        .split(':')
        .map((s) => parseInt(s, 10));
    if (range.length !== 2) {
        throw new Error('range input must be two integers separated by a ":"');
    }
    let start;
    if (input.startsWith('[')) {
        start = range[0];
    }
    else if (input.startsWith('(')) {
        start = range[0] + 1;
    }
    else {
        throw new Error('range input must begin with "[" (inclusive) or "(" (exclusive)');
    }
    let end;
    if (input.endsWith(']')) {
        end = range[1];
    }
    else if (input.endsWith(')')) {
        end = range[1] - 1;
    }
    else {
        throw new Error('range input must end with "]" (inclusive) or ")" (exclusive)');
    }
    return { start, end };
};
exports.parseIntRange = parseIntRange;
function argBuilder(parser) {
    return (name, args) => (Object.assign(Object.assign({ name }, args), { required: true, parse: parser }));
}
exports.argBuilder = argBuilder;
exports.Flags = {
    intRangeArray: command_1.flags.build({
        parse: (s) => s.split(',').map((r) => (0, exports.parseIntRange)(r.trim())),
        helpValue: "'[0:1], [1:2]'",
    }),
    addressArray: command_1.flags.build({
        parse: exports.parseAddressArray,
        helpValue: '\'["0xb7ef0985bdb4f19460A29d9829aA1514B181C4CD", "0x47e172f6cfb6c7d01c1574fa3e2be7cc73269d95"]\'',
    }),
    address: command_1.flags.build({
        parse: parseAddress,
        description: 'Account Address',
        helpValue: '0xc1912fEE45d61C87Cc5EA59DaE31190FFFFf232d',
    }),
    ecdsaPublicKey: command_1.flags.build({
        parse: parseEcdsaPublicKey,
        description: 'ECDSA Public Key',
        helpValue: '0x',
    }),
    blsPublicKey: command_1.flags.build({
        parse: parseBlsPublicKey,
        description: 'BLS Public Key',
        helpValue: '0x',
    }),
    blsProofOfPossession: command_1.flags.build({
        parse: parseBlsProofOfPossession,
        description: 'BLS Proof-of-Possession',
        helpValue: '0x',
    }),
    contract: command_1.flags.build({
        parse: parseCoreContract,
        description: 'Core Contract Name',
        helpValue: `${contractkit_1.CeloContract.BlockchainParameters}`,
    }),
    contractsArray: command_1.flags.build({
        parse: parseArray(parseCoreContract),
        description: 'Array of Registered Core Contracts',
        helpValue: `\'["${contractkit_1.CeloContract.BlockchainParameters}", "${contractkit_1.CeloContract.Governance}", "${contractkit_1.CeloContract.Validators}"]\'`,
    }),
    phoneNumber: command_1.flags.build({
        parse: parsePhoneNumber,
        description: 'Phone Number in E164 Format',
        helpValue: '+14152223333',
    }),
    proofOfPossession: command_1.flags.build({
        parse: parseProofOfPossession,
        description: 'Proof-of-Possession',
        helpValue: '0x',
    }),
    url: command_1.flags.build({
        parse: parseUrl,
        description: 'URL',
        helpValue: 'https://www.celo.org',
    }),
    wei: command_1.flags.build({
        parse: parseWei,
        description: 'Token value without decimals',
        helpValue: '10000000000000000000000',
    }),
};
exports.Args = {
    address: argBuilder(parseAddress),
    file: argBuilder(exports.parsePath),
    // TODO: Check that the file path is possible
    newFile: argBuilder((x) => x),
};
//# sourceMappingURL=command.js.map